/*
 * plugin.cpp
 * Copyright 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *     Steve Grubb <sgrubb@redhat.com>
 *     Junji Kanemaru <junji.kanemaru@linuon.com>
 */
#include <dlfcn.h>
#include "plugin.h"
#include "plugin_api.h"
#include "fault.h"

using namespace std;

/**
 * Constructors/Destructors
 */
Plugin::Plugin () { 
    path = NULL; 
    pluginInterface = NULL; 
    libPtr = NULL;
}

Plugin::~Plugin() {
    unload();
}

/**
 * Methods
 */

string Plugin::getName() const {
    if (pluginInterface == NULL) {
        throw APP_FAULT(NOT_INITIALIZED, "Plugin not initialized");
    }

    return string(pluginInterface->getPluginName());
}

PluginType Plugin::getType() const {
    if (pluginInterface == NULL) {
        throw APP_FAULT(NOT_INITIALIZED, "Plugin not initialized");
    }

    return pluginInterface->getPluginType();
}

void Plugin::load(const char* fname) {
    libPtr = dlopen(fname, RTLD_LAZY);
    if (libPtr == NULL) {
        throw SYSTEM_FAULT(LOAD_FAILED, dlerror());
    }
    else {
        __PluginEntry entry = 
                (__PluginEntry)dlsym(libPtr, PLUGIN_ENTRY_SYMBIL_NAME);

        /* Check dl library error with dlerror() function */
        char* errmsg = dlerror();
        if (errmsg != NULL || entry == NULL) {
            dlclose(libPtr);
            throw SYSTEM_FAULT(LOAD_FAILED, errmsg);
        }

        pluginInterface = (*entry)();
        if (pluginInterface == NULL) {
            dlclose(libPtr);
            throw APP_FAULT(INTERFACE_ERROR, "Plugin interface error");
        }
    }
}

void Plugin::unload() {
    if (libPtr != NULL) {
        dlclose(libPtr);
    }
    libPtr = NULL;
    pluginInterface = NULL;
    path = NULL;
}

